#define SERIAL_PORT "/dev/ttyS0"
#define BAUDRATE B38400

#define CMD_MAGIC 0xa7


#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

int serial_init(){
	struct termios oldtio, newtio;
	int serial_fd;
	serial_fd = open(SERIAL_PORT, O_RDWR | O_NOCTTY);
	if(serial_fd < 0){ perror(SERIAL_PORT " could not be opened"); exit(-1); }
	/*set up serial attributes */
	tcgetattr(serial_fd, &oldtio); //save current attr
	bzero(&newtio, sizeof(newtio));
        /*
         * BAUDRATE: Set bps rate.
         * CRTSCTS : output hardware flow control (only used if the cable has
         * all necessary lines. see sect. 7 of Serial-HOWTO)
         * CS8 : 8n1 (8bit, no parity, 1 stopbit)
         * CLOCAL : local connection, no modem control
         * CREAD : enable receiveing characters
         */
	newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
       /*
        * IGNPAR : ignore bytes with parity errors
        * ICRNL : map CR to NL (otherwise a CR input on the other computer
        * will not terminate input)
        * otherwise make device raw (no other input processing)
        */
	newtio.c_iflag = IGNPAR | ICRNL;
       /*
        * Raw output
        */
	newtio.c_oflag = 0;
         /*
         * initailize all control characters
         * default values can be found in /usr/include/termios.h, and are given
         * in the comments, but we don't need them here
         */
	newtio.c_cc[VINTR] = 0; /* Ctrl-c */
	newtio.c_cc[VQUIT] = 0; /* Ctrl-\ */
	newtio.c_cc[VERASE] = 0; /* del */
	newtio.c_cc[VKILL] = 0; /* @ */
	newtio.c_cc[VEOF] = 4; /* Ctrl-d */
	newtio.c_cc[VTIME] = 0; /* inter-character timer unused */
	newtio.c_cc[VMIN] = 1; /* blocking read until 1 character arrives */
	newtio.c_cc[VSWTC] = 0; /* '\0' */
	newtio.c_cc[VSTART] = 0; /* Ctrl-q */
	newtio.c_cc[VSTOP] = 0; /* Ctrl-s */
	newtio.c_cc[VSUSP] = 0; /* Ctrl-z */
	newtio.c_cc[VEOL] = 0; /* '\0' */
	newtio.c_cc[VREPRINT] = 0; /* Ctrl-r */
	newtio.c_cc[VDISCARD] = 0; /* Ctrl-u */
	newtio.c_cc[VWERASE] = 0; /* Ctrl-w */
	newtio.c_cc[VLNEXT] = 0; /* Ctrl-v */
	newtio.c_cc[VEOL2] = 0; /* '\0' */
        /*
         * now clean the modem line and activate the settings for the port
         */
	tcflush(serial_fd, TCIFLUSH);
	tcsetattr(serial_fd, TCSANOW, &newtio);
	return serial_fd;
}
